==> Slurm-Sacctmgr-1.0.1.tar.gz: Slurm-Sacctmgr-1.0.1/lib/Slurm/Sacctmgr/Association.pm <==
#!/usr/local/bin/perl
#
#Part of Slurm::Sacctmgr: Perl wrapper for Slurm's sacctmgr cmd
#Represents an Association

package Slurm::Sacctmgr::Association;
use strict;
use warnings;
use base qw(Slurm::Sacctmgr::EntityBaseListable);
use Carp qw(carp croak);

#-------------------------------------------------------------------
#	Globals
#-------------------------------------------------------------------

#-------------------------------------------------------------------
#	Accessors
#-------------------------------------------------------------------

my @rw_accessors = qw(
	account
	cluster
	defaultqos
	fairshare
	grpcpumins
	grpcpus
	grpjobs
	grpnodes
	grpsubmitjobs
	grpwall
	id
	lft
	maxcpumins
	maxcpus
	maxjobs
	maxnodes
	maxsubmitjobs
	maxwall
	parentid
	parentname
	partition
	qos
	rgt
	user
);

__PACKAGE__->mk_accessors(@rw_accessors);

#-------------------------------------------------------------------
#	Overloaded methods
#-------------------------------------------------------------------

sub _rw_fields($)
{	my $class = shift;
	return [ @rw_accessors ];
}

sub _sacctmgr_name_field($)
{	my $class = shift;
	return 'account';
}
 
sub _sacctmgr_fields_in_order($)
{	my $class = shift;
	return [ @rw_accessors ];
}

sub _my_sacctmgr_where_clause($)
#OVerloaded to match on cluster, account, user, and partition
#Returns a where clause hash that should return the current entity.
{       my $obj = shift;
        croak "Must be called as an instance method at "
                unless $obj && ref($obj);

	my @fields = qw(cluster account user partition);
	my ($fld, $val, $meth);
	my $where = {};
	foreach $fld (@fields)
	{	$meth = $fld;
		$val = $obj->$meth;
		$val = '' unless defined $val;
		$where->{$fld} = $val;
	}
	return $where;
}

#-------------------------------------------------------------------
#	Constructors, etc
#-------------------------------------------------------------------

#All inherited

1;
__END__

=head1 NAME

Slurm::Sacctmgr::Association

=head1 SYNOPSIS

  use Slurm::Sacctmgr::Association;
  use Slurm::Sacctmgr;

  my $sacctmgr = new Slurm::Sacctmgr;
  my $list = Slurm::Sacctmgr::Association->sacctmgr_list($sacctmgr);

  #List all associations known to sacctmgr
  foreach $association (@$list)
  {	#association is a Slurm::Sacctmgr::Association object
  	$uname = $association->user;
	$cluster = $association->cluster;
	$partition = $association->partition;
	$cpumins = $association->grpcpumins;
	print "$uname $cluster $partition $cpumins\n";
  }

  ...


=head1 DESCRIPTION

Represents a Association entity in Slurm::Sacctmgr.  Together with an
instance of  B<Slurm::Sacctmgr>, this class allows one to issue
commands to the Slurm B<sacctmgr> command to add, delete, list,
show, and modify sacctmgr entities of type "association".  

The B<Slurm::Sacctmgr> class provides a Perlish wrapper around
the actual Slurm B<sacctmgr> command, thus the methods provided
by this class largely map quite straightforwardly onto B<sacctmgr>
commands.  When using the B<sacctmgr_list> method for this class, the 
results from the sacctmgr command is automatically parsed and presented
as objects of this class.

Objects of this class contain the following data members:

=over 4

=item B<account>: the account for the association

=item B<cluster>: the cluster for the association

=item B<user>: the user for the association

=item B<partition>: the partition for the association

=item B<defaultqos>: the default QoS for this association

=item B<fairshare>: the fairshare number for this association

=item B<grpcpumins>: the total number of CPU minutes usable by jobs with this association

=item B<grpcpus>: the maximum number of CPUs running jobs in this association can have allocated

=item B<grpjobs>: the maximum number of running jobs this association can have

=item B<grpnodes>: the maximum number of nodes jobs running in this association can have allocated

=item B<grpsubmitjobs>: the maximum number of jobs (pending or running) the association can have at a time

=item B<grpwall>: the maximum wall clock time jobs running in the association can have

=item B<id>: the id number for this association

=item B<lft>, B<rgt>: index numbers, all children of this association will have an B<lft> between this B<lft> and B<rgt>

=item B<maxcpumins>: the maximum number of CPU minutes each job in this association can use

=item B<maxcpus>: the maximum number of CPUs each job in this association can use

=item B<maxjobs>: the maximum number of jobs each user in this association can run at one time

=item B<maxsubmitjobs>: the maximum number of jobs (pending or running) each user in this association can have at a given time

=item B<maxwall>: the maximum wall clock time each job in this association can use

=item B<parentid>: the id number of the parent of this association

=item B<parent>: the name of the parent of this association

=item B<qos>: the QoSes that jobs in this association can use

=back

and these are the fields that will be set by the B<sacctmgr_list> and similar
methods.  When filtering results, you can use any fields recognized by the
B<sacctmgr> command, and when adding/modifying records you can provide any
fields recognized by the command.

Most functionality is provided via the base classes, 

=over 4

=item B<Slurm::Sacctmgr::EntityBaseListable>

=item B<Slurm::Sacctmgr::EntityBase>

=back

As a result, the whole B<Slurm::Sacctmgr::*> family (for different entity types)
all have a very similar association interface (basically the differences are what
operations sacctmgr allows on different entity types, and how the results
of list/show operations get parsed).  

Sacctmgr does NOT allow the direct addition, deletion and/or modification of 
associations --- instead you should add, delete, or modify the user or account, etc.
entities to which the associations attach.  

As such, the only real method provided by this package is
B<sacctmgr_list($sacctmgr, %where)>.  This will return a list reference of all 
associations matching the "where" clause.  
The "where" clause is a list of zero or more key => value pairs 
specifying the conditions.  Only exact matching is supported, and keys cannot
be usefully repeated (the "where" clause is treated as a hash, so later keys
simply overwrite earlier keys).  You can use any parameter the Slurm
B<sacctmgr> command allows you to filter on.  If no 
key => value pairs are given, all entities of this type are returned.

=head2 EXPORT

Nothing.  Pure OO interface.

=head1 SEE ALSO

=over 4

=item B<Slurm::Sacctmgr::EntityBase>

=item B<Slurm::Sacctmgr::EntityBaseListable>

=item B<Slurm::Sacctmgr::EntityBaseAddDel>

=item B<Slurm::Sacctmgr::EntityBaseModifiable>

=item B<Slurm::Sacctmgr::EntityBaseRW>

=item B<sacctmgr> man page

=back

=head1 AUTHOR

Tom Payerle, payerle@umd.edu

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2014-2016 by the University of Maryland.

This program is free software; you can redistribute it and/or modify it
under the terms of the the Artistic License (2.0). You may obtain a
copy of the full license at:

L<http://www.perlfoundation.org/artistic_license_2_0>

Any use, modification, and distribution of the Standard or Modified
Versions is governed by this Artistic License. By using, modifying or
distributing the Package, you accept this license. Do not use, modify,
or distribute the Package, if you do not accept this license.

If your Modified Version has been derived from a Modified Version made
by someone other than you, you are nevertheless required to ensure that
your Modified Version complies with the requirements of this license.

This license does not grant you the right to use any trademark, service
mark, tradename, or logo of the Copyright Holder.

This license includes the non-exclusive, worldwide, free-of-charge
patent license to make, have made, use, offer to sell, sell, import and
otherwise transfer the Package with respect to any patent claims
licensable by the Copyright Holder that are necessarily infringed by the
Package. If you institute patent litigation (including a cross-claim or
counterclaim) against any party alleging that the Package constitutes
direct or contributory patent infringement, then this Artistic License
to you shall terminate on the date that such litigation is filed.

Disclaimer of Warranty: THE PACKAGE IS PROVIDED BY THE COPYRIGHT HOLDER
AND CONTRIBUTORS "AS IS' AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES.
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE, OR NON-INFRINGEMENT ARE DISCLAIMED TO THE EXTENT PERMITTED BY
YOUR LOCAL LAW. UNLESS REQUIRED BY LAW, NO COPYRIGHT HOLDER OR
CONTRIBUTOR WILL BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, OR
CONSEQUENTIAL DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THE PACKAGE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

==> Slurm-Sacctmgr-1.1.0.tar.gz: Slurm-Sacctmgr-1.1.0/lib/Slurm/Sacctmgr/Association.pm <==
#!/usr/local/bin/perl
#
#Part of Slurm::Sacctmgr: Perl wrapper for Slurm's sacctmgr cmd
#Represents an Association

package Slurm::Sacctmgr::Association;
use strict;
use warnings;
use base qw(Slurm::Sacctmgr::EntityBaseListable);
use Carp qw(carp croak);

#-------------------------------------------------------------------
#	Globals
#-------------------------------------------------------------------

#-------------------------------------------------------------------
#	Accessors
#-------------------------------------------------------------------

#Accessors common to all versions of slurm
my @common_accessors = qw(
	account
	cluster
	defaultqos
	fairshare
	grpjobs
	grpsubmitjobs
	grpwall
	id
	lft
	maxjobs
	maxsubmitjobs
	maxwall
	parentid
	parentname
	partition
	qos
	rgt
	user
);

#Accessors in pre-TRES (< 15.x.y) versions of Slurm
my @preTRES_accessors = qw(
	grpcpumins
	grpcpurunmins
	grpcpus
	grpnodes
	maxcpumins
	maxcpus
	maxnodes
);

#Accessors in post-TRES versions of Slurm
my @postTRES_accessors = qw(
	grptresmins
	grptresrunmins
	grptres
	maxtresmins
	maxtres
	maxtrespernode
);

my @all_accessors = (
	@common_accessors,
	@preTRES_accessors,
	@postTRES_accessors,
);

my @simple_accessors = ( @common_accessors );


__PACKAGE__->mk_accessors(@simple_accessors);

#-------------	Special accessors; handle TRES and non-TRES fields

__PACKAGE__->mk_tres_nontres_accessors('grptres', 'grpcpus' => 'cpu', 'grpnodes' => 'node' );
__PACKAGE__->mk_tres_nontres_accessors('grptresmins', 'grpcpumins' => 'cpu' );
__PACKAGE__->mk_tres_nontres_accessors('grptresrunmins', 'grpcpurunmins' => 'cpu' );
__PACKAGE__->mk_tres_nontres_accessors('maxtres', 'maxcpus' => 'cpu', 'maxnodes' => 'node' );
__PACKAGE__->mk_tres_nontres_accessors('maxtresmins', 'maxcpumins' => 'cpu' );


#-------------------------------------------------------------------
#	Overloaded methods
#-------------------------------------------------------------------

sub _rw_fields($)
{	my $class = shift;
	return [ @all_accessors ];
}

#Do NOT overload this, should never get used.
#Associations cannot be specified with one single field
#sub _sacctmgr_name_field($)
#{	my $class = shift;
#	return 'account';
#}
 
sub _sacctmgr_fields_in_order($$)
{	my $class = shift;
	my $sacctmgr = shift;
	my @fields = @common_accessors;
	if ( $sacctmgr->sacctmgr_cmd_supports('trackable_resources') )
	{	push @fields, @postTRES_accessors;
	} else
	{	push @fields, @preTRES_accessors;
	}

	return [ @fields ];
}

sub _my_sacctmgr_where_clause($)
#OVerloaded to match on cluster, account, user, and partition
#Returns a where clause hash that should return the current entity.
{       my $obj = shift;
        croak "Must be called as an instance method at "
                unless $obj && ref($obj);

	my @fields = qw(cluster account user partition);
	my ($fld, $val, $meth);
	my $where = {};
	foreach $fld (@fields)
	{	$meth = $fld;
		$val = $obj->$meth;
		$val = '' unless defined $val;
		$where->{$fld} = $val;
	}
	return $where;
}

#-------------------------------------------------------------------
#	Constructors, etc
#-------------------------------------------------------------------

#All inherited

1;
__END__

=head1 NAME

Slurm::Sacctmgr::Association

=head1 SYNOPSIS

  use Slurm::Sacctmgr::Association;
  use Slurm::Sacctmgr;

  my $sacctmgr = new Slurm::Sacctmgr;
  my $list = Slurm::Sacctmgr::Association->sacctmgr_list($sacctmgr);

  #List all associations known to sacctmgr
  foreach $association (@$list)
  {	#association is a Slurm::Sacctmgr::Association object
  	$uname = $association->user;
	$cluster = $association->cluster;
	$partition = $association->partition;
	$cpumins = $association->grpcpumins;
	print "$uname $cluster $partition $cpumins\n";
  }

  ...


=head1 DESCRIPTION

Represents a Association entity in Slurm::Sacctmgr.  Together with an
instance of  B<Slurm::Sacctmgr>, this class allows one to issue
commands to the Slurm B<sacctmgr> command list or
show entities of type "association".  

The B<Slurm::Sacctmgr> class provides a Perlish wrapper around
the actual Slurm B<sacctmgr> command, thus the methods provided
by this class largely map quite straightforwardly onto B<sacctmgr>
commands.  When using the B<sacctmgr_list> method for this class, the 
results from the sacctmgr command is automatically parsed and presented
as objects of this class.

Objects of this class contain the following data members:

=over 4

=item B<account>: the account for the association

=item B<cluster>: the cluster for the association

=item B<user>: the user for the association

=item B<partition>: the partition for the association

=item B<defaultqos>: the default QoS for this association

=item B<fairshare>: the fairshare number for this association

=item B<grpcpumins>: the total number of CPU minutes usable by jobs with this association

=item B<grpcpus>: the maximum number of CPUs running jobs in this association can have allocated

=item B<grpjobs>: the maximum number of running jobs this association can have

=item B<grpnodes>: the maximum number of nodes jobs running in this association can have allocated

=item B<grpsubmitjobs>: the maximum number of jobs (pending or running) the association can have at a time

=item B<grpwall>: the maximum wall clock time jobs running in the association can have

=item B<id>: the id number for this association

=item B<lft>, B<rgt>: index numbers, all children of this association will have an B<lft> between this B<lft> and B<rgt>

=item B<maxcpumins>: the maximum number of CPU minutes each job in this association can use

=item B<maxcpus>: the maximum number of CPUs each job in this association can use

=item B<maxjobs>: the maximum number of jobs each user in this association can run at one time

=item B<maxsubmitjobs>: the maximum number of jobs (pending or running) each user in this association can have at a given time

=item B<maxwall>: the maximum wall clock time each job in this association can use

=item B<parentid>: the id number of the parent of this association

=item B<parent>: the name of the parent of this association

=item B<qos>: the QoSes that jobs in this association can use

=back

and these are the fields that will be set by the B<sacctmgr_list> and similar
methods.  When filtering results, you can use any fields recognized by the
B<sacctmgr> command.

Most functionality is provided via the base classes, 

=over 4

=item B<Slurm::Sacctmgr::EntityBaseListable>

=item B<Slurm::Sacctmgr::EntityBase>

=back

As a result, the whole B<Slurm::Sacctmgr::*> family (for different entity types)
all have a very similar association interface (basically the differences are what
operations sacctmgr allows on different entity types, and how the results
of list/show operations get parsed).  

Sacctmgr does NOT allow the direct addition, deletion and/or modification of 
associations --- instead you should add, delete, or modify the user or account, etc.
entities to which the associations attach.  

As such, the only real method provided by this package is
B<sacctmgr_list($sacctmgr, %where)>.  This will return a list reference of all 
associations matching the "where" clause.  
The "where" clause is a list of zero or more key => value pairs 
specifying the conditions.  Only exact matching is supported, and keys cannot
be usefully repeated (the "where" clause is treated as a hash, so later keys
simply overwrite earlier keys).  You can use any parameter the Slurm
B<sacctmgr> command allows you to filter on.  If no 
key => value pairs are given, all entities of this type are returned.

=head2 EXPORT

Nothing.  Pure OO interface.

=head1 SEE ALSO

=over 4

=item B<Slurm::Sacctmgr::EntityBase>

=item B<Slurm::Sacctmgr::EntityBaseListable>

=item B<sacctmgr> man page

=back

=head1 AUTHOR

Tom Payerle, payerle@umd.edu

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2014-2016 by the University of Maryland.

This program is free software; you can redistribute it and/or modify it
under the terms of the the Artistic License (2.0). You may obtain a
copy of the full license at:

L<http://www.perlfoundation.org/artistic_license_2_0>

Any use, modification, and distribution of the Standard or Modified
Versions is governed by this Artistic License. By using, modifying or
distributing the Package, you accept this license. Do not use, modify,
or distribute the Package, if you do not accept this license.

If your Modified Version has been derived from a Modified Version made
by someone other than you, you are nevertheless required to ensure that
your Modified Version complies with the requirements of this license.

This license does not grant you the right to use any trademark, service
mark, tradename, or logo of the Copyright Holder.

This license includes the non-exclusive, worldwide, free-of-charge
patent license to make, have made, use, offer to sell, sell, import and
otherwise transfer the Package with respect to any patent claims
licensable by the Copyright Holder that are necessarily infringed by the
Package. If you institute patent litigation (including a cross-claim or
counterclaim) against any party alleging that the Package constitutes
direct or contributory patent infringement, then this Artistic License
to you shall terminate on the date that such litigation is filed.

Disclaimer of Warranty: THE PACKAGE IS PROVIDED BY THE COPYRIGHT HOLDER
AND CONTRIBUTORS "AS IS' AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES.
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE, OR NON-INFRINGEMENT ARE DISCLAIMED TO THE EXTENT PERMITTED BY
YOUR LOCAL LAW. UNLESS REQUIRED BY LAW, NO COPYRIGHT HOLDER OR
CONTRIBUTOR WILL BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, OR
CONSEQUENTIAL DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THE PACKAGE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
